import Async, { CustomSignalClass, Signal } from "typescene-async";
import { Container } from "../";
import Block from "./Block";
import Component from "../Component";
import { EventHandlerCtor } from "../DOMEvents";
import { ComponentFactory, ValueOrAsync } from "../ComponentFactory";
import TextLabelFactory from "../TextLabelFactory";
import { ActionHandler } from "../DOMEvents";
/** Represents a row of tabs or pills */
export declare class NavList extends Block {
    /** Signal that is emitted when an item is activated, with the content of the item's `target` property, e.g. a URL string or Activity (class) from the `typescene-app` module; default connection only navigates to URLs but the Application constructor connects a new handler that calls `startActivity` */
    static ActivateTarget: CustomSignalClass<any, Signal<any>>;
    /** Signal that is emitted when a new item is included in a NavList instance, with the item's `target` property, for which a connected handler should return an observable boolean value indicating whether this value matches the currently active route/activity; implemented by Application in the `typescene-app` module using its `isActive` method, otherwise not connected by default */
    static QueryActiveTarget: CustomSignalClass<any, Signal<any>>;
    /** Create a navigation list component */
    constructor(navItems?: NavList.NavItem[], type?: NavList.Type, selectedIndex?: number, selectedKey?: string);
    /** Initialize a nav list factory with given values */
    static with: (values: NavList.Initializer) => ComponentFactory<NavList>;
    /** Initialize a nav list factory with given items */
    static withItems(navItems: NavList.NavItem[]): ComponentFactory<NavList>;
    /** Initialize with given (observable) values; returns this */
    initializeWith: (values: NavList.Initializer) => this;
    /** Nav labels, icons, and badges (observed) */
    navItems: NavList.NavItem[];
    /** Index of selected item (base 0), or -1 if no selection (observed/able) */
    selectedIndex: number;
    /** Key of selected item (observed/able) */
    selectedKey: string | undefined;
    /** Type of nav: tabs or pills, inline or stacked; default tabs (observed) */
    type: NavList.Type;
    /** Set to true to fill all horizontal space */
    justified: boolean;
    /** Select given nav item by index or key, and start associated activity if any; returns this */
    activate(selection: number | string): this;
    /** Returns an array of directly contained UI components (observable) */
    getChildren(): Component[];
    /** Signal emitted when the selected nav item changes */
    readonly NavChange: NavListSelectionSignal;
    private _ul;
    private _li;
    private _containerBlock;
}
export declare namespace NavList {
    /** Represents an item in a navigation list (tab or pill) */
    interface NavItem {
        /** Text label */
        label?: string | TextLabelFactory;
        /** Icon (see Label) */
        icon?: string;
        /** Space reserved for icon (see Label) */
        iconGutter?: number;
        /** Badge text */
        badge?: string;
        /** Key (string ID) of the item, used with selectedKey */
        key?: string;
        /** Container to be displayed below tabs/pills when this item is selected */
        container?: Container;
        /** URL/path string, Activity instance, or Activity class that is activated when this item is selected; also in reverse: the item will be automatically selected if/when its target is active (i.e. URL/path matches, or activity/class matches current activity or one of its parent activities) */
        target?: any;
    }
}
export declare namespace NavList {
    /** NavList display type */
    enum Type {
        /** Display as a row of tabs */
        Tabs = 0,
        /** Display as a row of pills */
        Pills = 1,
        /** Display as a list of pills */
        StackedPills = 2,
    }
}
export declare namespace NavList {
    /** Initializer for .with({ ... }) */
    interface Initializer extends Block.Initializer {
        /** Property initializer */
        navItems: ValueOrAsync<NavList.NavItem[]>;
        /** Property initializer */
        selectedIndex?: ValueOrAsync<number>;
        /** Property initializer */
        selectedKey?: ValueOrAsync<string>;
        /** Property initializer */
        type?: ValueOrAsync<NavList.Type>;
        /** Property initializer */
        justified?: ValueOrAsync<boolean>;
        /** Signal initializer: method name or handler */
        NavChange?: string | NavListSelectionHandler;
    }
}
/** Event data that is emitted when a navlist's selection changes */
export interface NavListSelectionEvent {
    /** The navlist component itself */
    navList: NavList;
    /** The new selection index */
    selectedIndex: number;
    /** The new selection key */
    selectedKey?: string;
}
/** Signal that is emitted when a navlist's selection changes */
export interface NavListSelectionSignal extends Async.CustomSignalClass<NavListSelectionEvent, Async.Signal<NavListSelectionEvent>> {
}
/** [implementation] */
export interface NavListSelectionHandler extends ActionHandler<NavListSelectionEvent, NavListSelectionSignal> {
}
/** Constructor for a navlist selection change handler */
export declare var NavListSelectionHandler: EventHandlerCtor<NavListSelectionEvent, NavListSelectionHandler>;
export default NavList;
